/*
 * The MIT License
 *
 * Copyright 2020 Charles University
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.cuni.mff.java.cv07plus;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

public class Shell {
    private final List<ShellCommand> commands;
    private Path currentDir;

    private class HelpCommand implements ShellCommand {

        @Override
        public String help() {
            return "Prints out all commands and their help";
        }

        @Override
        public String name() {
            return "help";
        }

        @Override
        public void execute(String... args) {
            System.out.println("All commands");
            System.out.println("============");
            for (ShellCommand command: commands) {
                System.out.println(command.name());
                System.out.println("  " + command.help());
                System.out.println();
            }
        }
    }

    private class DirCommand implements ShellCommand {

        @Override
        public String help() {
            return "Lists the current directory";
        }

        @Override
        public String name() {
            return "dir";
        }

        @Override
        public void execute(String... args) {
            try {
                Files.list(currentDir).forEach(System.out::println);
            } catch (IOException e) {
                System.out.println("IOException occurred");
            }
        }
    }

    private class CatCommand implements ShellCommand {

        @Override
        public String help() {
            return "Prints out a file";
        }

        @Override
        public String name() {
            return "cat";
        }

        @Override
        public void execute(String... args) {
            if (args.length < 1) {
                System.out.println("no file to print out");
                return;
            }
            try (BufferedReader reader = Files.newBufferedReader(Path.of(args[0]))) {
                int c;
                while ((c = reader.read()) != -1) {
                    System.out.write(c);
                }
                System.out.println();
            } catch (IOException ex) {
                System.out.println("Cannot read the file");
            }
        }
    }

    // other commands similarly


    public Shell() {
        currentDir = Path.of(".");
        commands = new ArrayList<>();
        commands.add(new HelpCommand());
        commands.add(new DirCommand());
        commands.add(new CatCommand());
    }

    private ShellCommand getCommand(String name) {
        for (ShellCommand command: commands) {  // HashMap<String, ShellCommand> will be a more appropriate type for the "commands"
            if (command.name().equals(name)) {
                return command;
            }
        }
        return null;
    }

    public void run() throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
        String line;
        System.out.print("> ");
        while ((line = reader.readLine()) != null) {
            String[] lineElements = line.split(" +");
            if (lineElements.length == 0) {
                System.out.println("  Bad command\n");
                continue;
            }
            ShellCommand command = getCommand(lineElements[0]);
            if (command == null) {
                System.out.println("  Unknown command\n");
                continue;
            }
            String[] args = new String[lineElements.length - 1];
            for (int i = 1; i < lineElements.length; i++) { // better: use System.arraycopy()
                args[i - 1] = lineElements[i];
            }
            command.execute(args);
            System.out.print("> ");
        }
    }

    public static void main(String[] args) {
        try {
            Shell shell = new Shell();
            shell.run();
        } catch (IOException ex) {
            System.out.println("IOException occurred");
        }
    }
}